@@ -288,6 +288,51 @@ static char *proc_comm_dup(const char *str)
 	return comm;
 }
 
+/*
+ *  get_proc_self_stat_field()
+ *	find nth field of /proc/$PID/stat data. This works around
+ *	the problem that the comm field can contain spaces and
+ *	multiple ) so sscanf on this field won't work.  The returned
+ *	pointer is the start of the Nth field and it is up to the
+ *	caller to determine the end of the field
+ */
+static const char *get_proc_self_stat_field(const char *buf, const int num)
+{
+	const char *ptr = buf, *comm_end;
+	int n;
+
+	if (num < 1 || !buf || !*buf)
+		return NULL;
+	if (num == 1)
+		return buf;
+	if (num == 2)
+		return strstr(buf, "(");
+
+	comm_end = NULL;
+	for (ptr = buf; *ptr; ptr++) {
+		if (*ptr == ')')
+			comm_end = ptr;
+	}
+	if (!comm_end)
+		return NULL;
+	comm_end++;
+	n = num - 2;
+
+	ptr = comm_end;
+	while (*ptr) {
+		while (*ptr && *ptr == ' ')
+			ptr++;
+		n--;
+		if (n <= 0)
+			break;
+		while (*ptr && *ptr != ' ')
+			ptr++;
+	}
+
+	return ptr;
+}
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  secs_to_str()
  *	report seconds in different units.
@@ -309,6 +354,7 @@ static char *secs_to_str(const double secs)
 		(unsigned int)s, (unsigned int)fract, second_scales[i].ch);
 	return buf;
 }
+#endif
 
 /*
  *  get_username()
@@ -588,8 +634,13 @@ static void get_extra(const pid_t pid, proc_info_t * const info)
 		}
 	}
 
-	if (fscanf(fp, "%*d %*s %*s %*d %*d %*d %ld", &dev) == 1)
-		info->tty = (dev_t)dev;
+	info->tty = (dev_t)0;
+	if (fgets(buffer, sizeof(buffer) - 1, fp) != NULL) {
+		const char *ptr = get_proc_self_stat_field(buffer, 7);
+
+		if (ptr && sscanf(ptr , "%ld", &dev) == 1)
+			info->tty = (dev_t)dev;
+	}
 	(void)fclose(fp);
 }
 
@@ -629,6 +680,7 @@ ret:
 
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  get_parent_pid()
  *	get parent pid and set is_thread to true if process
@@ -679,6 +731,7 @@ static pid_t get_parent_pid(const pid_t pid, bool * const is_thread)
 
 	return ppid;
 }
+#endif
 
 /*
  *  sane_proc_pid_info()
@@ -823,6 +876,7 @@ static void print_heading(void)
 		(opt_flags & OPT_GLYPH) ? " " : "");
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  row_increment()
  *	bump row increment and re-print heading if required
@@ -837,6 +891,7 @@ static void row_increment(void)
 		row = 2;
 	}
 }
+#endif
 
 /*
  *  timeval_to_double()
@@ -858,6 +913,7 @@ static void proc_info_get_timeval(const pid_t pid, struct timeval * const tv)
 	unsigned long jiffies;
 	char path[PATH_MAX];
 	char buffer[4096];
+	const char *ptr;
 	double uptime_secs, secs;
 	struct timeval now;
 	ssize_t n;
@@ -867,11 +923,12 @@ static void proc_info_get_timeval(const pid_t pid, struct timeval * const tv)
 	fd = open("/proc/uptime", O_RDONLY);
 	if (fd < 0)
 		return;
-	n = read(fd, buffer, sizeof(buffer));
+	n = read(fd, buffer, sizeof(buffer) - 1);
 	if (n <= 0) {
 		(void)close(fd);
 		return;
 	}
+	buffer[n] = '\0';
 	(void)close(fd);
 	n = sscanf(buffer, "%lg", &uptime_secs);
 	if (n != 1)
@@ -882,14 +939,18 @@ static void proc_info_get_timeval(const pid_t pid, struct timeval * const tv)
 	fd = open(path, O_RDONLY);
 	if (fd < 0)
 		return;
-	n = read(fd, buffer, sizeof(buffer));
+	n = read(fd, buffer, sizeof(buffer) - 1);
 	if (n <= 0) {
 		(void)close(fd);
 		return;
 	}
+	buffer[n] = '\0';
 	(void)close(fd);
-	n = sscanf(buffer, "%*d %*s %*c %*d %*d %*d %*d %*d %*u %*u %*u %*u "
-		"%*u %*u %*u %*u %*u %*d %*d %*d %*d %llu", &starttime);
+
+	ptr = get_proc_self_stat_field(buffer, 22);
+	if (!ptr)
+		return;
+	n = sscanf(ptr, "%llu", &starttime);
 	if (n != 1)
 		return;
 
@@ -936,6 +997,7 @@ static inline unsigned int proc_name_hash(const char *str)
 	return hash % MAX_PIDS;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  proc_stats_account()
  *	perform per process accounting
@@ -980,6 +1042,7 @@ static void proc_stats_account(const pid_t pid, const event_t event)
 	stats->next = proc_stats[h];
 	proc_stats[h] = stats;
 }
+#endif
 
 /*
  *  stats_cmp()
@@ -1157,6 +1220,7 @@ static proc_info_t *proc_info_get(const pid_t pid)
 	return info;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  proc_info_free()
  *	free cached process info and remove from hash table
@@ -1179,6 +1243,7 @@ static void proc_info_free(const pid_t pid)
 		info = info->next;
 	}
 }
+#endif
 
 /*
  *   proc_info_unload()
@@ -1200,6 +1265,7 @@ static void proc_info_unload(void)
 	}
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  proc_info_update()
  *	update process name, for example, if exec has occurred
@@ -1225,6 +1291,7 @@ static proc_info_t const *proc_info_update(const pid_t pid)
 
 	return info;
 }
+#endif
 
 /*
  *   proc_info_add()
@@ -1332,6 +1399,7 @@ static int proc_info_load(void)
 	return 0;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 /*
  *  extra_info()
  *	format up extra process information if selected
@@ -1355,6 +1423,7 @@ static char *extra_info(const uid_t uid)
 
 	return buf;
 }
+#endif
 
 /*
  *  handle_sig()
@@ -1488,8 +1557,10 @@ static int monitor(const int sock)
 			struct tm tm;
 			char when[10];
 			time_t now;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 			pid_t pid, ppid;
 			bool is_thread;
+#endif
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 			struct timeval tv;
